@@ -1,4 +1,4 @@
-/* Copyright (c) 1997-2020
+/* Copyright (c) 1997-2023
    Ewgenij Gawrilow, Michael Joswig, and the polymake team
    Technische Universität Berlin, Germany
    https://polymake.org
@@ -15,8 +15,7 @@
 --------------------------------------------------------------------------------
 */
 
-#ifndef POLYMAKE_INTERNAL_COMPARATORS_BASIC_DEFS_H
-#define POLYMAKE_INTERNAL_COMPARATORS_BASIC_DEFS_H
+#pragma once
 
 #include "polymake/internal/operations_basic_defs.h"
 
@@ -360,7 +359,6 @@ struct are_comparable_via<T1, T2, Comparator, decltype(std::declval<Comparator>(
 
 } // end namespace pm
 
-#endif // POLYMAKE_INTERNAL_COMPARATORS_BASIC_DEFS_H
 
 // Local Variables:
 // mode:C++
