@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2023
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -55,9 +56,9 @@ canonicalize_point_configuration(GenericVector<TVector>& V)
 template <typename TMatrix> inline
 void canonicalize_point_configuration(GenericMatrix<TMatrix>& M)
 {
-   Set<int> neg;
-   int i = 0;
-   for (auto r=entire(rows(M)); !r.at_end();  ++r, ++i) {
+   Set<Int> neg;
+   Int i = 0;
+   for (auto r = entire(rows(M)); !r.at_end();  ++r, ++i) {
       if ( r->top()[0] < 0 )
          neg.push_back(i);
       else
@@ -66,8 +67,8 @@ void canonicalize_point_configuration(GenericMatrix<TMatrix>& M)
    M = M.minor(~neg,All);
 }
 
-FunctionTemplate4perl("canonicalize_point_configuration(Vector&) : void");
-FunctionTemplate4perl("canonicalize_point_configuration(Matrix&) : void");
+FunctionTemplate4perl("canonicalize_point_configuration(Vector&)");
+FunctionTemplate4perl("canonicalize_point_configuration(Matrix&)");
 
 } }
 
