@@ -1,6 +1,7 @@
-/* Copyright (c) 1997-2018
-   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-   http://www.polymake.org
+/* Copyright (c) 1997-2023
+   Ewgenij Gawrilow, Michael Joswig, and the polymake team
+   Technische Universität Berlin, Germany
+   https://polymake.org
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -24,15 +25,15 @@
 namespace polymake { namespace polytope {
 
 template <typename Scalar, typename MatrixType>
-perl::Object weighted_digraph_polyhedron(const GenericMatrix<MatrixType, Scalar>& W)
+BigObject weighted_digraph_polyhedron(const GenericMatrix<MatrixType, Scalar>& W)
 {
-   const int k(W.rows());
+   const Int k = W.rows();
    if (W.cols() != k)
          throw std::runtime_error("weighted_digraph_polyhedron: non-square matrix");
 
-   ListMatrix< SparseVector<Scalar> > Ineq;
-   for (int i=0; i<k; ++i)
-      for (int j=0; j<k; ++j) {
+   ListMatrix<SparseVector<Scalar>> Ineq;
+   for (Int i = 0; i < k; ++i)
+      for (Int j = 0; j < k; ++j) {
          if (isfinite(W.top()(i,j)) && (i!=j || W.top()(i,j) != 0)) {
             SparseVector<Scalar> this_ineq(k+1); // initialized as zero vector
             this_ineq[0]=W.top()(i,j);
@@ -40,20 +41,26 @@ perl::Object weighted_digraph_polyhedron(const GenericMatrix<MatrixType, Scalar>
             Ineq /= this_ineq;
          }
       }
-  
-   perl::Object p_out("Polytope", mlist<Scalar>());
-   p_out.set_description() << "Weighted digraph polyhedron" <<endl;
-   p_out.take("INEQUALITIES") << Ineq;
-   Matrix<Scalar> wd(W);
-   p_out.attach("WEIGHTED_DIGRAPH") << wd;
+
+   BigObject p_out("Polytope", mlist<Scalar>(), "INEQUALITIES", Ineq);
+   p_out.attach("WEIGHTED_DIGRAPH") << Matrix<Scalar>(W);
+   p_out.set_description() << "Weighted digraph polyhedron" << endl;
    return p_out;
 }
 
 UserFunctionTemplate4perl("# @category Producing a polytope from graphs"
-                          "# Weighted digraph polyhedron of a directed graph with a weight function."
+                          "# Weighted digraph polyhedron of a directed graph with a weight function as studied in"
+                          "#   Joswig, Loho: Weighted digraph polyhedra and tropical cones, LAA (2016)."
                           "# The graph and the weight function are combined into a matrix."
                           "# @param Matrix encoding weighted digraph"
-                          "# @return polytope::Polytope",
+                          "# @return polytope::Polytope"
+                          "# @example This digraph has two nodes and a single arc (with weight 2)."
+                          "# > $enc = new Matrix([[0,2],[\"inf\",0]]);"
+                          "# > $Q = weighted_digraph_polyhedron($enc);"
+                          "# > print $Q->FACETS;"
+                          "# | 2 -1 1"
+                          "# | 1 0 0"
+                          "# These are the one defining inequality and the trivial inequality, which contains the far face.",
                           "weighted_digraph_polyhedron<Scalar>(Matrix<Scalar,_>)");
 } }
 
