@@ -1,6 +1,7 @@
-#  Copyright (c) 1997-2018
-#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Berlin, Germany)
-#  http://www.polymake.org
+#  Copyright (c) 1997-2023
+#  Ewgenij Gawrilow, Michael Joswig, and the polymake team
+#  Technische Universität Berlin, Germany
+#  https://polymake.org
 #
 #  This program is free software; you can redistribute it and/or modify it
 #  under the terms of the GNU General Public License as published by the
@@ -15,7 +16,7 @@
 
 use strict;
 use feature 'state';
-use vars qw( $dirmode $group $permmask $root $ext_root $buildroot $config_file %ConfigFlags $buildmode
+use vars qw( $dirmode $group $permmask $root $ext_root $builddir $config_file %ConfigFlags $buildmode
              $xsmod @callable @fakelibs );
 use POSIX qw ( :fcntl_h read write lseek );
 use File::Path;
@@ -24,8 +25,8 @@ use Config;
 use Getopt::Long qw( GetOptions :config require_order no_ignore_case );
 
 sub collect_arglist {
-   my $list=shift;
-   @$list=@_;
+   my $list = shift;
+   @$list = @_;
    while (@ARGV && $ARGV[0] !~ /^-/) {
       push @$list, shift @ARGV;
    }
@@ -33,7 +34,7 @@ sub collect_arglist {
 
 unless (GetOptions( 'root=s' => \$root,
                     'extroot=s' => \$ext_root,
-                    'buildroot=s' => \$buildroot,
+                    'builddir=s' => \$builddir,
                     'config=s' => \$config_file,
                     'mode=s' => \$buildmode,
                     'group=s' => \$group,
@@ -44,51 +45,51 @@ unless (GetOptions( 'root=s' => \$root,
         !@ARGV &&
         defined($config_file) &&
         defined($root) &&
-        defined($buildroot) &&
+        defined($builddir) &&
         defined($buildmode) &&
         -f $config_file &&
         -d $root &&
-        -d "$buildroot/$buildmode" &&
+        -d "$builddir/$buildmode" &&
         (defined($ext_root) ? -d $ext_root && !defined($xsmod) && !@callable && !@fakelibs
                             : defined($xsmod) && -f $xsmod)) {
    die <<".";
-usage: $0 --root PATH --buildroot PATH --config PATH/config.ninja [ --extroot PATH --extconfig PATH/config.ninja ]
+usage: $0 --root PATH --builddir PATH --config PATH/config.ninja [ --extroot PATH --extconfig PATH/config.ninja ]
           --buildmode Opt|Debug [ --group GROUP ] [ --perms MASK ] [ --xsmod | --callable | --fakelib shared_module ... ]
 .
 }
 
-if (defined $group) {
-   my $gid=getgrnam($group)
+if (defined($group)) {
+   my $gid = getgrnam($group)
      or die "$0: unknown group '$group'\n";
-   $group=$gid;
+   $group = $gid;
 }
 
 do "$root/support/install_utils.pl";
 die $@ if $@;
-%ConfigFlags=load_config_file($config_file, $root);
-my @BundledExts= $ConfigFlags{BundledExts} =~ /(\S+)/g;
+%ConfigFlags = load_config_file($config_file, $root);
+my @BundledExts = $ConfigFlags{BundledExts} =~ /(\S+)/g;
 
-my @default_exclude_for_copy=qw( \. \.\. \.\#.* .*~ \.git.* \.noexport );
+my @default_exclude_for_copy = qw( \. \.\. \.\#.* .*~ \.git.* \.noexport );
 
 # these variables may be used in sourced extension scripts
-use vars qw($InstallTop $InstallInc $InstallArch $InstallBin $InstallLib $InstallDoc $builddir $ExtTop $ExtArch);
+use vars qw($InstallTop $InstallInc $InstallArch $InstallBin $InstallLib $buildtop $ExtTop $ExtArch);
 
-foreach (qw(InstallTop InstallInc InstallArch InstallBin InstallLib InstallDoc)) {
+foreach (qw(InstallTop InstallInc InstallArch InstallBin InstallLib)) {
    no strict 'refs';
-   ${$_}=$ConfigFlags{$_};
+   ${$_} = $ConfigFlags{$_};
 }
-$builddir="$buildroot/$buildmode";
-my $ext_name= $ext_root && basename($ext_root);
+$buildtop = "$builddir/$buildmode";
+my $ext_name = $ext_root && basename($ext_root);
 
 my $destdir = $ENV{DESTDIR} // $ConfigFlags{DESTDIR};
 $destdir =~ s{/$}{};
 if (length($destdir)) {
-   foreach ($InstallTop, $InstallInc, $InstallArch, $InstallBin, $InstallLib, $InstallDoc) {
+   foreach ($InstallTop, $InstallInc, $InstallArch, $InstallBin, $InstallLib) {
       substr($_,0,0) .= $destdir;
    }
 }
 
-my $perlxpath="perlx/$Config::Config{version}/$Config::Config{archname}";
+my $perlxpath = "perlx/$Config::Config{version}/$Config::Config{archname}";
 
 if ($buildmode eq "San") {
    load_sanitizer_flags();
@@ -99,12 +100,12 @@ if (defined $permmask) {
       die "--perms must allow full access to the file owner\n";
    }
    umask 0;
-   $dirmode=0777 & $permmask;
+   $dirmode = 0777 & $permmask;
 } else {
-   $permmask=0777&~umask;
+   $permmask = 0777&~umask;
 }
 
-if (defined $ext_root) {
+if (defined($ext_root)) {
    install_extension();
 } else {
    install_core();
@@ -112,25 +113,25 @@ if (defined $ext_root) {
 
 sub install_core {
    make_dir($InstallTop);
-   foreach my $subdir (qw(demo perllib resources scripts xml)) {
+   foreach my $subdir (qw(demo perllib resources scripts upgrades)) {
       copy_dir("$root/$subdir", "$InstallTop/$subdir", clean_dir => 1);
    }
 
-   my $clean_dir=1;
+   my $clean_dir = 1;
    foreach my $file (qw(configure.pl.template generate_applib_fake.pl generate_ninja_targets.pl
                         generate_ninja_targets.pl.template generate_cpperl_modules.pl
                         guarded_compiler.pl install.pl install.pl.template install_utils.pl rules.ninja)) {
       copy_file("$root/support/$file", "$InstallTop/support/$file", clean_dir => $clean_dir);
-      $clean_dir=0;
+      $clean_dir = 0;
    }
 
    make_dir("$InstallTop/apps", clean_dir => 1);
    make_dir("$InstallTop/bundled", clean_dir => 1);
 
    foreach my $bundled (glob("$root/bundled/*")) {
-      my $ext_file="$bundled/polymake.ext";
+      my $ext_file = "$bundled/polymake.ext";
       copy_file($ext_file, $InstallTop.substr($ext_file, length($root)), clean_dir => 1);
-      if (-d (my $scripts="$root/bundled/scripts")) {
+      if (-d (my $scripts = "$root/bundled/scripts")) {
          copy_dir($scripts, $InstallTop.substr($scripts, length($root)));
       }
    }
@@ -148,8 +149,8 @@ sub install_core {
    copy_dir("$root/lib/core/skel", "$InstallTop/lib/core/skel", clean_dir => 1);
 
    foreach my $dir (glob_all_apps("include")) {
-      my ($app_name)= $dir =~ m{/apps/(\w+)/};
-      my $wrappers="$`/include/app-wrappers/polymake/$app_name";
+      my ($app_name) = $dir =~ m{/apps/(\w+)/};
+      my $wrappers = "$`/include/app-wrappers/polymake/$app_name";
       copy_dir($dir, "$InstallInc/polymake/$app_name",
                -e $wrappers ? (wrappers => $wrappers) : ());
    }
@@ -160,8 +161,7 @@ sub install_core {
    }
 
    foreach my $dir (glob_all_apps("src")) {
-      copy_dir($dir, $InstallTop.substr($dir, length($root)),
-               exclude => [ 'perl' ]);
+      copy_dir($dir, $InstallTop.substr($dir, length($root)));
    }
 
    make_dir($InstallArch);
@@ -178,14 +178,14 @@ sub install_core {
 
    -d $InstallLib || make_dir($InstallLib);
 
-   if (defined $xsmod) {
+   if (defined($xsmod)) {
       $xsmod =~ m{($perlxpath/auto/.*\.$Config::Config{dlext})$}o
         or die "$0: path of perl extension module does not match the expected pattern\n";
       copy_file($xsmod, "$InstallArch/$1", mode => 0555, clean_dir => 1);
    }
 
    foreach my $lib_file (@callable) {
-      my $to=$InstallLib."/".basename($lib_file);
+      my $to = $InstallLib."/".basename($lib_file);
       if (-l $lib_file) {
          copy_link($lib_file, $to);
       } else {
@@ -194,8 +194,8 @@ sub install_core {
    }
    make_dir("$InstallArch/lib", clean_dir => 1);
    foreach my $app_dir (glob("$root/apps/*")) {
-      my $app_mod=basename($app_dir).".$Config::Config{dlext}";
-      copy_file("$builddir/lib/$app_mod", "$InstallArch/lib/$app_mod", mode => 0555);
+      my $app_mod = basename($app_dir).".$Config::Config{dlext}";
+      copy_file("$buildtop/lib/$app_mod", "$InstallArch/lib/$app_mod", mode => 0555);
    }
 
    # These symlinks are used by the callable library bootstrap module.
@@ -208,8 +208,8 @@ sub install_core {
    if (@fakelibs) {
       my $stub_lib_name;
       foreach my $lib_file (@fakelibs) {
-         my $basename=basename($lib_file);
-         my $to="$InstallArch/lib/$basename";
+         my $basename = basename($lib_file);
+         my $to = "$InstallArch/lib/$basename";
          if (-l $lib_file) {
             copy_link($lib_file, $to);
             copy_link($lib_file, "$InstallLib/$basename");
@@ -222,12 +222,8 @@ sub install_core {
       }
    }
 
-   if (-f "$buildroot/doc/index.html") {
-      copy_dir("$buildroot/doc", $InstallDoc, clean_dir => 1);
-   }
-
    foreach my $bundled (@BundledExts) {
-      if (-f (my $inst_script="$root/bundled/$bundled/support/install.pl")) {
+      if (-f (my $inst_script = "$root/bundled/$bundled/support/install.pl")) {
          do $inst_script;
          die "installation script $inst_script failed: $@" if $@;
       }
@@ -235,10 +231,10 @@ sub install_core {
 }
 
 sub install_extension {
-   $ExtTop="$InstallTop/ext/$ext_name";
+   $ExtTop = "$InstallTop/ext/$ext_name";
    copy_file("$ext_root/polymake.ext", "$ExtTop/polymake.ext", clean_dir => 1);
 
-   foreach my $subdir (qw(resources scripts xml)) {
+   foreach my $subdir (qw(resources scripts upgrades)) {
       if (-d "$ext_root/$subdir") {
          copy_dir("$ext_root/$subdir", "$ExtTop/$subdir");
       }
@@ -250,8 +246,8 @@ sub install_extension {
    }
 
    foreach my $dir (glob("$ext_root/apps/*/include")) {
-      my ($app_name)= $dir =~ m{/apps/(\w+)/};
-      my $wrappers="$`/include/app-wrappers/polymake/$app_name";
+      my ($app_name) = $dir =~ m{/apps/(\w+)/};
+      my $wrappers = "$`/include/app-wrappers/polymake/$app_name";
       copy_dir($dir, "$ExtTop/include/polymake/$app_name",
                -e $wrappers ? (wrappers => $wrappers) : ());
    }
@@ -261,30 +257,30 @@ sub install_extension {
                exclude => [ 'perl' ]);
    }
 
-   $ExtArch="$InstallArch/ext/$ext_name";
-   copy_file("$buildroot/config.ninja", "$ExtArch/config.ninja", clean_dir => 1,
+   $ExtArch = "$InstallArch/ext/$ext_name";
+   copy_file("$builddir/config.ninja", "$ExtArch/config.ninja", clean_dir => 1,
              transform => \&transform_extension_config_file);
 
    make_dir("$ExtArch/lib");
    foreach my $app_dir (glob("$ext_root/apps/*")) {
-      my $app_mod=basename($app_dir).".$Config::Config{dlext}";
-      copy_file("$builddir/lib/$app_mod", "$ExtArch/lib/$app_mod", mode => 0555);
+      my $app_mod = basename($app_dir).".$Config::Config{dlext}";
+      copy_file("$buildtop/lib/$app_mod", "$ExtArch/lib/$app_mod", mode => 0555);
    }
 
-   if (-f (my $inst_script="$ext_root/support/install.pl")) {
+   if (-f (my $inst_script = "$ext_root/support/install.pl")) {
       do $inst_script;
       die "installation script $inst_script failed: $@" if $@;
    }
 }
 
 sub glob_all_apps {
-   my ($pattern)=@_;
+   my ($pattern) = @_;
    ( glob("$root/apps/*/$pattern"),
      map { glob("$root/bundled/$_/apps/*/$pattern") } @BundledExts )
 }
 
 sub copy_file {
-   my ($from, $to, %options)=@_;
+   my ($from, $to, %options) = @_;
    if (-e $from) {
       if (!-f $from and !-l $from) {
          die "$0: $from is neither a regular file nor a symbolic link\n";
@@ -301,8 +297,8 @@ sub copy_file {
 }
 
 sub copy_link {
-   my ($from, $to)=@_;
-   my $target=readlink($from);
+   my ($from, $to) = @_;
+   my $target = readlink($from);
    if (-e $to || -l $to) {
       unlink $to
         or die "can't remove old $to: $!\n";
@@ -312,18 +308,18 @@ sub copy_link {
 }
 
 sub rel_symlink {
-   my ($target, $link)=@_;
+   my ($target, $link) = @_;
    if (-e $link || -l $link) {
       unlink $link
         or die "$0: can't delete old $link: $!\n";
    }
-   my $rel_link=$link;
-   my $common_prefix=1;
+   my $rel_link = $link;
+   my $common_prefix = 1;
    while ($rel_link =~ s{^(/[^/]+)(?=/)}{}) {
-      if ($common_prefix && substr($target, 0, length($1)) eq $1) {
-         $target=substr($target, length($1));
+      if ($common_prefix && substr($target, 0, length($1)+1) eq "$1/") {
+         $target = substr($target, length($1));
       } else {
-         $common_prefix=0;
+         $common_prefix = 0;
          $target =~ s{^/?}{../};
       }
    }
@@ -333,34 +329,34 @@ sub rel_symlink {
 }
 
 sub compile_pattern {
-   my $expr=join('|', map { "(?:^$_\$)" } @_);
+   my $expr = join('|', map { "(?:^$_\$)" } @_);
    qr/$expr/;
 }
 
 sub exclude_pattern_for_copy {
-   my ($list)=@_;
+   my ($list) = @_;
    if (defined $list) {
       compile_pattern(@$list, @default_exclude_for_copy);
    } else {
-      state $default_pattern=compile_pattern(@default_exclude_for_copy);
+      state $default_pattern = compile_pattern(@default_exclude_for_copy);
    }
 }
 
-my $z1024=pack "x1024";
+my $z1024 = pack "x1024";
 
 sub copy {
-   my ($from, $to, %options)=@_;
+   my ($from, $to, %options) = @_;
    my $concat;
-   my $mode=$options{mode};
+   my $mode = $options{mode};
 
    if (-e $to) {
-      if (defined (my $conflict=$options{conflict})) {
+      if (defined(my $conflict = $options{conflict})) {
          foreach my $c (@$conflict) {
             if ($to =~ $c->[0]) {
                if ($c->[1] eq 'abort') {
                   die "$0: can't install $from: file $to already exists\n";
                } elsif ($c->[1] eq 'concat') {
-                  $concat=1;
+                  $concat = 1;
                } elsif ($c->[1] eq 'keep') {
                   return;
                } elsif ($c->[1] ne 'replace') {
@@ -374,30 +370,30 @@ sub copy {
          die "$0: can't remove old $to: $!\n";
       }
    }
-   my ($fmode, $mtime)=(stat $from)[2,9];
+   my ($fmode, $mtime) = (stat $from)[2, 9];
    if (defined $mode) {
       # verbatim copy: assuming binary file
-      my $in=POSIX::open $from, O_RDONLY;
+      my $in = POSIX::open $from, O_RDONLY;
       defined($in)
          or die "$0: can't read $from: $!\n";
-      my $dummy=O_WRONLY+O_CREAT+O_TRUNC; # bug in AutoLoader!
+      my $dummy = O_WRONLY + O_CREAT + O_TRUNC;  # bug in AutoLoader!
       my $out;
       if ($concat) {
          open $out, ">>$to";
       } else {
-         $out=creat $to, 0600;
+         $out = creat $to, 0600;
       }
       defined($out)
          or die "$0: can't create $to: $!\n";
 
       my $trailing_zeroes;
-      while ((my $size=read $in, $_, 1024)>0) {
-         if ($_ eq ($size==1024 ? $z1024 : pack("x$size"))) {
+      while ((my $size = read $in, $_, 1024) > 0) {
+         if ($_ eq ($size == 1024 ? $z1024 : pack("x$size"))) {
             lseek $out, $size, SEEK_CUR;
-            $trailing_zeroes=1;
+            $trailing_zeroes = 1;
          } else {
             write $out, $_, $size;
-            $trailing_zeroes=0;
+            $trailing_zeroes = 0;
          }
       }
       if ($trailing_zeroes) {
@@ -407,19 +403,19 @@ sub copy {
       POSIX::close $in;
       POSIX::close $out;
 
-      if (my $lib_id=$options{lib_id}) {
-         substr($lib_id, 0, length($destdir))="";
+      if (my $lib_id = $options{lib_id}) {
+         substr($lib_id, 0, length($destdir)) = "";
          system("install_name_tool -id $lib_id $to")
            and die "install_name_tool $to failed\n";
       }
    } else {
       # text file: turn on write protection
-      $mode=$fmode & 0555;
+      $mode = $fmode & 0555;
       local $/;
       open X, "<", $from
          or die "$0: can't read $from: $!\n";
-      $_=<X>; close X;
-      if (defined (my $transform=$options{transform})) {
+      $_ = <X>; close X;
+      if (defined (my $transform = $options{transform})) {
 	 &$transform;
       }
       open X, ">", $to
@@ -429,7 +425,7 @@ sub copy {
    $mode &= $permmask;
    chmod $mode, $to
      or die "can't modify access rights to $to: $!\n";
-   if (defined $group) {
+   if (defined($group)) {
       chown -1, $group, $to
         or die "$0: can't change group of $to: $!\n";
    }
@@ -438,8 +434,8 @@ sub copy {
 }
 
 sub make_dir {
-   my ($dir, %options)=@_;
-   my $clean=$options{clean_dir};
+   my ($dir, %options) = @_;
+   my $clean = $options{clean_dir};
 
    if (-e $dir) {
       if (-d _) {
@@ -447,19 +443,19 @@ sub make_dir {
             chmod $dirmode, $dir
                or die "$0: can't change mode of $dir: $!\n";
          }
-         if (defined $group and (stat _)[5] != $group) {
+         if (defined($group) and (stat _)[5] != $group) {
             chown -1, $group, $dir
               or die "$0: can't change group of $dir: $!\n";
          }
          if ($clean) {
             my $error;
-            if (defined( my $exclude=$options{exclude})) {
-               my $exclude_re=compile_pattern(@$exclude, qw(. ..));
+            if (defined(my $exclude = $options{exclude})) {
+               my $exclude_re = compile_pattern(@$exclude, qw(. ..));
                opendir my $D, $dir;
                foreach (readdir $D) {
                   if ($_ !~ $exclude_re) {
                      if (-d "$dir/$_") {
-                        File::Path::remove_tree("$dir/_", { error=>\$error });
+                        File::Path::remove_tree("$dir/_", { error => \$error });
                         if (@$error) {
                            die "$0: can't remove old files:\n", map { join(": ", %$_)."\n" } @$error;
                         }
@@ -470,7 +466,7 @@ sub make_dir {
                   }
                }
             } else {
-               File::Path::remove_tree($dir, { keep_root=>1, error=>\$error });
+               File::Path::remove_tree($dir, { keep_root => 1, error => \$error });
                if (@$error) {
                   die "$0: can't clean $dir:\n", map { join(": ", %$_)."\n" } @$error;
                }
@@ -487,10 +483,10 @@ sub make_dir {
 }
 
 sub copy_dir {
-   my ($src, $dst, %options)=@_;
-   my $wrappers=delete $options{wrappers};
-   my $clean=$options{clean_dir};
-   my $exclude_re=exclude_pattern_for_copy($options{exclude});
+   my ($src, $dst, %options) = @_;
+   my $wrappers = delete $options{wrappers};
+   my $clean = $options{clean_dir};
+   my $exclude_re = exclude_pattern_for_copy($options{exclude});
 
    if (opendir my $S, $src) {
       make_dir($dst, clean_dir => $clean);
@@ -500,24 +496,24 @@ sub copy_dir {
            or die "can't read $src/.noexport: $!\n";
          local $/="\n";
          while (<$noexport>) {
-            if (my ($file, $status)= /^(?!\#)(\S+)\s+(\S+)$/) {
+            if (my ($file, $status) = /^(?!\#)(\S+)\s+(\S+)$/) {
                if ($file =~ /[?*\[\]]/) {
                   $file =~ s/./\\./g;
                   $file =~ tr/?/./;
                   $file =~ s/\*/.*/g;
                   push @noexport_patterns, [ qr{^$file$}, $status ];
                } else {
-                  $noexport{$file}=$status;
+                  $noexport{$file} = $status;
                }
             }
          }
       }
       foreach my $f (grep { $_ !~ $exclude_re } readdir $S) {
-         my $noexport=$noexport{$f};
+         my $noexport = $noexport{$f};
          unless ($noexport) {
             foreach my $pat (@noexport_patterns) {
                if ($f =~ $pat->[0]) {
-                  $noexport=$pat->[1];
+                  $noexport = $pat->[1];
                   last;
                }
             }
@@ -525,7 +521,7 @@ sub copy_dir {
          if ($noexport) { 
             next if $noexport ne "local";
          }
-         my $src_f="$src/$f";
+         my $src_f = "$src/$f";
          if (-d $src_f) {
             copy_dir($src_f, "$dst/$f", %options);
          } elsif ($wrappers && -f "$wrappers/$f") {
@@ -546,7 +542,7 @@ sub install_bin_scripts {
    local $/;
    open S, "$root/perl/polymake"
      or die "can't read $root/perl/polymake: $!\n";
-   $_=<S>;
+   $_ = <S>;
    close S;
 
    if ($^O eq "darwin" && $ConfigFlags{ARCHFLAGS} =~ /-arch /) {
@@ -555,7 +551,7 @@ sub install_bin_scripts {
       s{^\#!/usr/bin/env perl}{#!$Config::Config{perlpath}}s;
    }
 
-   my $init_block=<<"---";
+   my $init_block = <<"---";
    \$InstallTop='$ConfigFlags{InstallTop}';
    \$InstallArch='$ConfigFlags{InstallArch}';
    \$Arch="$ConfigFlags{Arch}";
@@ -580,11 +576,11 @@ sub install_bin_scripts {
 
    # apply similar transformations to the configuration utility
 
-   my $Version=extract_polymake_version($root);
+   my $Version = extract_polymake_version($root);
 
    open S, "$root/perl/polymake-config"
      or die "can't read $root/perl/polymake-config: $!\n";
-   $_=<S>;
+   $_ = <S>;
    close S;
 
    if ($^O eq "darwin" && $ConfigFlags{ARCHFLAGS} =~ /-arch /) {
@@ -611,23 +607,23 @@ sub install_bin_scripts {
 }
 
 sub load_sanitizer_flags {
-   open my $BB, "<", "$builddir/build.ninja"
-      or die "can't read $builddir/build.ninja: $!\n";
+   open my $BB, "<", "$buildtop/build.ninja"
+      or die "can't read $buildtop/build.ninja: $!\n";
    while (<$BB>) {
       chomp;
       if (/^\s*PERLSANITIZE\s*=\s*/) {
          $ConfigFlags{PERLSANITIZE}=$';
       } elsif (/^\s*perlxpath\s*=\s*/) {
          if ($perlxpath ne $') {
-            die "$builddir/build.ninja does not match the current running environment; expected perlxpath=$perlxpath\n";
+            die "$buildtop/build.ninja does not match the current running environment; expected perlxpath=$perlxpath\n";
          }
-      } elsif (/^\s*buildroot\s*=\s*/) {
-         if ($buildroot ne $') {
-            die "$builddir/build.ninja does not match the current running environment; expected buildroot=$buildroot\n";
+      } elsif (/^\s*builddir\s*=\s*/) {
+         if ($builddir ne $') {
+            die "$buildtop/build.ninja does not match the current running environment; expected builddir=$builddir\n";
          }
       }
    }
-   $ConfigFlags{PERLSANITIZE} or die "$builddir/build.ninja does not contain PERLSANITIZE flags\n";
+   $ConfigFlags{PERLSANITIZE} or die "$buildtop/build.ninja does not contain PERLSANITIZE flags\n";
 }
 
 sub transform_core_config_file {
@@ -636,7 +632,7 @@ sub transform_core_config_file {
    s{^\s* root \s*= \K .*}{$ConfigFlags{InstallTop}}xm;
    s{^\s* core\.includes \s*= \K .*}{-I$ConfigFlags{InstallInc}}xm;
 
-   my $external_includes= $ConfigFlags{ExternalHeaders} =~ /\S/ ? "-I$ConfigFlags{InstallInc}/polymake/external" : "";
+   my $external_includes = $ConfigFlags{ExternalHeaders} =~ /\S/ ? "-I$ConfigFlags{InstallInc}/polymake/external" : "";
    s{^\s* app\.includes \s*= \K .*}{$external_includes}xm;
 
    s{^(?=\s*Arch\s*=)}{PERL = $Config::Config{perlpath}\n}m;
@@ -651,7 +647,6 @@ sub transform_core_config_file {
 }
 
 sub transform_extension_config_file {
-   
    s{^\s* root \s*= \K .*}{$ConfigFlags{InstallTop}}xm;
    s{^\s* extroot \s*= \K .*}{$ConfigFlags{InstallTop}/ext/$ext_name}xm;
    s{^\s* app\.includes \s*= \K .*}{-I\${extroot}/include \${super.app.includes}}xm;
